/*
 * (c) Copyright 2002 PADL Software Pty Ltd
 * To anyone who acknowledges that this file is provided "AS IS"
 * without any express or implied warranty:
 *                 permission to use, copy, modify, and distribute this
 * file for any purpose is hereby granted without fee, provided that
 * the above copyright notices and this notice appears in all source
 * code copies, and that the name PADL Software Pty Ltd not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  PADL Software Pty Ltd
 * makes no representation about the suitability of this software for
 * any purpose.
 *
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/nameser.h>
#include <resolv.h>
#include <ldap.h>

#include <NetInfo/system.h>
#include <NetInfo/dsattribute.h>
#include <NetInfo/dsrecord.h>
#include <NetInfo/dsengine.h>

int main(int argc, char *argv[])
{
	char hname[MAXHOSTNAMELEN];
	char *dn = NULL;
	dsdata *key = NULL, *val = NULL;
	dsengine *e = NULL;
	char path[MAXPATHLEN], nipath[1024];
	dsattribute *a = NULL;
	dsrecord *r = NULL;
	u_int32_t dsid, rc, index, i;
	char *tag = (argc < 2) ? "network" : argv[1];

	if (argc > 3) {
		fprintf(stderr, "%s: Usage: binddnsdomain [<tag>] [<suffix>]\n", argv[0]);
		exit(1);
	}

	snprintf(path, sizeof(path), "/var/db/netinfo/%s.nidb", tag);

	if (gethostname(hname, sizeof(hname)) < 0) {
		fprintf(stderr, "%s: gethostname() failed\n", argv[0]);
		exit(1);
	}

	if (argc < 3) {
		if ((_res.options & RES_INIT) == 0 && res_init () == -1) {
			fprintf(stderr, "%s: could not initialize resolver\n", argv[0]);
			exit(1);
		}
	
		if ((rc = ldap_domain2dn(_res.defdname, &dn)) != LDAP_SUCCESS) {
			fprintf(stderr, "%s: could not map default domain: %s\n", argv[0], ldap_err2string(rc));
			goto out;
		}
	} else {
		dn = strdup(argv[2]);
	}

	snprintf(nipath, sizeof(nipath), "/machines/%s", hname);

	if ((rc = dsengine_open(&e, path, DSSTORE_FLAGS_ACCESS_READWRITE)) != DSStatusOK) {
		fprintf(stderr, "%s: could not open datastore %s: %s\n", argv[0], path, dsstatus_message(rc));
		goto out;
	}

	if ((rc = dsengine_netinfo_string_pathmatch(e, 0, nipath, &dsid)) != DSStatusOK) {
		fprintf(stderr, "%s: could not find %s: %s\n", argv[0], nipath, dsstatus_message(rc));
		goto out;
	}

	if ((rc = dsengine_fetch(e, dsid, &r)) != DSStatusOK) {
		fprintf(stderr, "%s: could not fetch record ID %d: %s\n", argv[0], dsid, dsstatus_message(rc));
		goto out;
	}

	key = cstring_to_dsdata("serves");
	a = dsrecord_attribute(r, key, SELECT_ATTRIBUTE);
	if (a == NULL) {
		fprintf(stderr, "%s: %s has not serves property\n", argv[0], nipath);
		goto out;
	}
	index = IndexNull;
	for (i = 0; i < a->count; i++) {
		if (IsStringDataType(a->value[i]->type) && 
			(strncmp(a->value[i]->data, "./", 2) == 0) &&
			(strcmp(a->value[i]->data + 2, tag) == 0)) {
			index = i;
			break;
		}
	}
	if (index == IndexNull) {
		fprintf(stderr, "%s: %s does not have ./%s in serves property\n", argv[0], nipath, tag);
		goto out;
	}
	dsdata_release(key);
	dsattribute_release(a);

	key = cstring_to_dsdata("suffix");
	dsrecord_remove_key(r, key, SELECT_ATTRIBUTE);
	a = dsattribute_new(key);
	val = cstring_to_dsdata(dn);
	for (i = 0; i < index; i++) {
		dsdata *x = cstring_to_dsdata("");
		/* Pad with empty values. */
		dsattribute_insert(a, x, i);
		dsdata_release(x);
	}
	dsattribute_insert(a, val, index);
	dsrecord_append_attribute(r, a, SELECT_ATTRIBUTE);

	if ((rc = dsengine_save_attribute(e, r, a, SELECT_ATTRIBUTE)) != DSStatusOK) {
		fprintf(stderr, "%s: could not save attribute %s: %s\n", argv[0], dsdata_to_cstring(key), dsstatus_message(rc));
		goto out;
	}
	printf("Bound suffix %s to %s on tag %s.\n", dn, nipath, tag);

out:
	dsdata_release(key);
	dsdata_release(val);
	dsrecord_release(r);
	dsattribute_release(a);
	free(dn);
	dsengine_close(e);

	exit((rc != DSStatusOK));
	return((rc != DSStatusOK));
}

